package com.wissamfawaz;

import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.function.Predicate;
import java.util.stream.Collectors;

public class Main {

	public static void main(String[] args) {
		Employee[] employees = {
			new Employee("Elie", "Azar", 2500, "IT"),
			new Employee("Elias", "Azar", 4000, "Multimedia"),
			new Employee("Mirna", "Zweif", 6000, "Multimedia"),
			new Employee("Ghasan", "Stefan", 5000, "IT")
		};
		
		List<Employee> list = Arrays.asList(employees);
		list.stream()
			.forEach(System.out::println);
		Predicate<Employee> fourToSixK = e -> e.getSalary()>=4000 && e.getSalary()<=6000;
		System.out.printf("Employees earning between 4K and 6K: %n");
		list.stream()
			.filter(fourToSixK)
			.sorted(Comparator.comparing(Employee::getSalary))
			.forEach(System.out::println);
		
		System.out.printf("First employee earning between 4K and 6K: %s%n", 
				list.stream().filter(fourToSixK).findFirst().get());
		
		Function<Employee, String> byLast = Employee::getLast;
		Function<Employee, String> byFirst = Employee::getFirst;
		
		Comparator<Employee> byLastThenFirst = Comparator.comparing(byLast).thenComparing(byFirst);
		System.out.printf("Employees sorted by last then first names: %n");
		list.stream()
			.sorted(byLastThenFirst)
			.forEach(System.out::println);
		
		System.out.printf("List of unique last names: %s%n", 
				list.stream().map(Employee::getLast).distinct().sorted().collect(Collectors.toList()));
		
		Map<String, List<Employee>> groupingByDept = 
				list.stream()
					.collect(Collectors.groupingBy(Employee::getDepartment, Collectors.toList()));
		groupingByDept.forEach(
				(dept, empsPerDept) -> {
					System.out.printf("%s:%n", dept);
					empsPerDept.forEach(System.out::println);
				});
		
		Map<String, Long> countPerDept = 
				list.stream()
					.collect(Collectors.groupingBy(Employee::getDepartment, Collectors.counting()));
		countPerDept.forEach(
				(dept, count) -> System.out.printf("There are %d members in %s%n", count, dept));
		
		System.out.printf("Avg salary: %f%n", 
				list.stream()
					.mapToDouble(Employee::getSalary)
					.average().getAsDouble());
		
		
		
		
		
		
		
		
		
		
		
		
		

	}

}
